@@ -1,4 +1,4 @@
-/* A Bison parser, made by GNU Bison 3.8.1.  */
+/* A Bison parser, made by GNU Bison 3.8.2.  */
 
 /* Bison interface for Yacc-like parsers in C
 
@@ -117,9 +117,12 @@ extern int yydebug;
     LEX_AUTOJOIN_ON_KICK = 318,    /* LEX_AUTOJOIN_ON_KICK  */
     LEX_IGNORE_CAPAB = 319,        /* LEX_IGNORE_CAPAB  */
     LEX_RECONN_TIMER = 320,        /* LEX_RECONN_TIMER  */
-    LEX_BOOL = 321,                /* LEX_BOOL  */
-    LEX_INT = 322,                 /* LEX_INT  */
-    LEX_STRING = 323               /* LEX_STRING  */
+    LEX_SASL_USERNAME = 321,       /* LEX_SASL_USERNAME  */
+    LEX_SASL_PASSWORD = 322,       /* LEX_SASL_PASSWORD  */
+    LEX_SASL_MECHANISM = 323,      /* LEX_SASL_MECHANISM  */
+    LEX_BOOL = 324,                /* LEX_BOOL  */
+    LEX_INT = 325,                 /* LEX_INT  */
+    LEX_STRING = 326               /* LEX_STRING  */
   };
   typedef enum yytokentype yytoken_kind_t;
 #endif
@@ -191,22 +194,25 @@ extern int yydebug;
 #define LEX_AUTOJOIN_ON_KICK 318
 #define LEX_IGNORE_CAPAB 319
 #define LEX_RECONN_TIMER 320
-#define LEX_BOOL 321
-#define LEX_INT 322
-#define LEX_STRING 323
+#define LEX_SASL_USERNAME 321
+#define LEX_SASL_PASSWORD 322
+#define LEX_SASL_MECHANISM 323
+#define LEX_BOOL 324
+#define LEX_INT 325
+#define LEX_STRING 326
 
 /* Value type.  */
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 union YYSTYPE
 {
-#line 73 "conf.y"
+#line 74 "conf.y"
 
 	int number;
 	char *string;
 	void *list;
 	struct tuple *tuple;
 
-#line 210 "conf.h"
+#line 216 "conf.h"
 
 };
 typedef union YYSTYPE YYSTYPE;
