@@ -14,11 +14,13 @@
 #ifndef DEFAULTS_H
 #define DEFAULTS_H
 
+#include "irc.h"
+
 #define DEFAULT_BACKLOG 1
 #define DEFAULT_ALWAYS_BACKLOG 0
 #define DEFAULT_BL_MSG_ONLY 0
 #define DEFAULT_BACKLOG_LINES 10
-#define DEFAULT_BACKLOG_NO_TIMESTAMP 0
+#define DEFAULT_BACKLOG_TIMESTAMP BLTSTime
 #define DEFAULT_BLRESET_ON_TALK 0
 #define DEFAULT_BLRESET_CONNECTION 0
 #define DEFAULT_LOG 1
@@ -27,5 +29,6 @@
 #define DEFAULT_LOG_LEVEL LOG_INFO
 #define DEFAULT_LOG_FORMAT "%u/%n/%Y-%m/%c.%d.log"
 #define DEFAULT_BIP_USE_NOTICE 0
+#define DEFAULT_RECONN_TIMER 120
 
 #endif /* DEFAULTS_H */
