@@ -2,7 +2,8 @@
  * $Id: irc.h,v 1.43 2005/04/21 06:58:50 nohar Exp $
  *
  * This file is part of the bip project
- * Copyright (C) 2004 2005 Arnaud Cornet and Loïc Gomez
+ * Copyright (C) 2004,2005 Arnaud Cornet
+ * Copyright (C) 2004,2005,2022 Loïc Gomez
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -76,12 +77,12 @@ struct bipuser {
 	char *default_realname;
 
 	/* backlog options */
-	int backlog:1;
+	char backlog;
 	int backlog_lines;
-	int always_backlog:1;
-	int bl_msg_only:1;
-	int blreset_on_talk:1;
-	int blreset_connection:1;
+	char always_backlog;
+	char bl_msg_only;
+	char blreset_on_talk;
+	char blreset_connection;
 
 	enum BLTimestamp backlog_timestamp;
 
@@ -92,11 +93,10 @@ struct bipuser {
 #endif
 
 	hash_t connections;
-	int in_use:1; /* for mark and sweep on reload */
+	char in_use; /* for mark and sweep on reload */
 };
 
-struct network
-{
+struct network {
 	char *name;
 #ifdef HAVE_LIBSSL
 	int ssl;
@@ -106,12 +106,15 @@ struct network
 	struct server *serverv;
 };
 
+#define SASL_AUTH_EXTERNAL 1
+#define SASL_AUTH_PLAIN 2
+
 struct link {
-	char *name;	/* id */
+	char *name; /* id */
 
 	/** link live data **/
 	struct link_server *l_server;
-	int l_clientc;
+	unsigned int l_clientc;
 	struct link_client **l_clientv;
 
 	struct log *log;
@@ -133,15 +136,15 @@ struct link {
 
 	/** link options */
 
-	int follow_nick:1;
-	int ignore_first_nick:1;
-	int autojoin_on_kick:1;
-	int ignore_server_capab:1;
+	char follow_nick;
+	char ignore_first_nick;
+	char autojoin_on_kick;
+	char ignore_server_capab;
 	list_t on_connect_send;
 	char *no_client_away_msg;
 	char *away_nick;
-	hash_t chan_infos;		/* channels we want */
-	list_t chan_infos_order;	/* for order only */
+	hash_t chan_infos;	 /* channels we want */
+	list_t chan_infos_order; /* for order only */
 
 	struct bipuser *user;
 
@@ -153,6 +156,9 @@ struct link {
 	char *username;
 	char *realname;
 	char *s_password;
+	char *sasl_username;
+	char *sasl_password;
+	int sasl_mechanism;
 	char *connect_nick;
 
 	/* socket creation info */
@@ -161,7 +167,7 @@ struct link {
 
 #ifdef HAVE_LIBSSL
 	int ssl_check_mode;
-	STACK_OF(X509) *untrusted_certs;
+	STACK_OF(X509) * untrusted_certs;
 #endif
 	int in_use; /* for mark and sweep on reload */
 };
@@ -182,9 +188,9 @@ struct link_any {
 
 #define IRCC_NONE (0)
 #define IRCC_NICK (1)
-#define IRCC_USER (1<<1)
-#define IRCC_PASS (1<<2)
-#define IRCC_READY (IRCC_NICK|IRCC_PASS|IRCC_USER)
+#define IRCC_USER (1 << 1)
+#define IRCC_PASS (1 << 2)
+#define IRCC_READY (IRCC_NICK | IRCC_PASS | IRCC_USER)
 
 struct link_client {
 	struct link_connection _link_c;
@@ -218,7 +224,7 @@ struct chan_info {
 	int backlog;
 };
 
-#define chan_info_new() bip_calloc(sizeof(struct chan_info), 1)
+#define chan_info_new() bip_calloc(sizeof(struct chan_info), (size_t)1)
 
 struct link_server {
 	struct link_connection _link_c;
@@ -229,11 +235,11 @@ struct link_server {
 	hash_t channels;
 
 	char *user_mode;
-	int user_mode_len;
+	size_t user_mode_len;
 
 	/* init stuff */
-	int lag;
-	int laginit_ts;
+	unsigned lag;
+	time_t laginit_ts;
 	int lagtest_timeout;
 
 	/* chanmodes */
@@ -265,12 +271,11 @@ void bip_init(bip_t *bip);
 struct link_client *irc_client_new(void);
 struct link_server *irc_server_new(struct link *link, connection_t *conn);
 void irc_server_free(struct link_server *is);
-struct client *client_new();
+struct client *client_new(void);
 void irc_main(bip_t *);
-int ischannel(char p);
 void irc_client_close(struct link_client *);
 void irc_client_free(struct link_client *);
-struct link *irc_link_new();
+struct link *irc_link_new(void);
 void link_kill(bip_t *bip, struct link *);
 void unbind_from_link(struct link_client *ic);
 char *nick_from_ircmask(const char *mask);
@@ -278,4 +283,3 @@ void irc_cli_backlog(struct link_client *ic, int hours);
 #define BIP_FAKEMASK "!bip@bip.bip.bip"
 
 #endif
-
