@@ -2,7 +2,8 @@
  * $Id: log.c,v 1.56 2005/04/21 06:58:50 nohar Exp $
  *
  * This file is part of the bip project
- * Copyright (C) 2004 Arnaud Cornet and Loïc Gomez
+ * Copyright (C) 2004 Arnaud Cornet
+ * Copyright (C) 2004,2022 Loïc Gomez
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,7 +20,10 @@
 #include <stdio.h>
 #include <string.h>
 
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wstrict-prototypes"
 extern int errno;
+#pragma GCC diagnostic pop
 extern int log_level;
 extern char *conf_log_root;
 extern char *conf_log_format;
@@ -27,8 +31,8 @@ extern int conf_log;
 
 extern FILE *conf_global_log_file;
 
-static int _log_write(log_t *logdata, logstore_t *lf, const char *d,
-		const char *str);
+static size_t _log_write(log_t *logdata, logstore_t *lf, const char *d,
+			 const char *str);
 static char *_log_wrap(const char *dest, const char *line);
 void logfile_free(logfile_t *lf);
 void log_drop(log_t *log, const char *storename);
@@ -49,16 +53,15 @@ int check_dir(char *filename, int is_fatal)
 		if (err) {
 			if (is_fatal)
 				fatal("mkdir(%s) %s", filename,
-						strerror(errno));
+				      strerror(errno));
 			mylog(LOG_ERROR, "mkdir(%s) %s", filename,
-					strerror(errno));
+			      strerror(errno));
 			return 1;
 		}
 	} else if (err) {
 		if (is_fatal)
 			fatal("stat(%s) %s", filename, strerror(errno));
-		mylog(LOG_ERROR, "stat(%s) %s", filename,
-				strerror(errno));
+		mylog(LOG_ERROR, "stat(%s) %s", filename, strerror(errno));
 		return 1;
 	} else if (!(statbuf.st_mode & S_IFDIR)) {
 		if (is_fatal)
@@ -72,9 +75,10 @@ int check_dir(char *filename, int is_fatal)
 
 int check_dir_r(char *dirname)
 {
-	int pos, count = 0;
+	size_t count = 0;
 	char *dir, *tmp;
-	int len = strlen(dirname);
+	size_t len = strlen(dirname);
+	size_t pos;
 
 	mylog(LOG_DEBUGVERB, "Recursive check of %s engaged", dirname);
 	tmp = dirname;
@@ -83,7 +87,7 @@ int check_dir_r(char *dirname)
 		int slash_ok = 1;
 		while (*tmp == '/') {
 			if (slash_ok) {
-				strncpy(dir + count, "/", 2);
+				strncpy(dir + count, "/", (size_t)2);
 				count++;
 				slash_ok = 0;
 			}
@@ -94,7 +98,7 @@ int check_dir_r(char *dirname)
 		tmp += pos;
 		count += pos;
 		*(dir + count) = '\0';
-		mylog(LOG_DEBUGVERB,"check_dir_r: %s", dir);
+		mylog(LOG_DEBUGVERB, "check_dir_r: %s", dir);
 		if (check_dir(dir, 0)) {
 			free(dir);
 			return 1;
@@ -109,7 +113,8 @@ void strtolower(char *str)
 	char *c;
 
 	for (c = str; *c != '\0'; c++)
-		*c = tolower(*c);
+		*c = (char)tolower(
+			*c); // tolower()->int but should be safe to cast
 }
 
 /*
@@ -120,10 +125,10 @@ void strtolower(char *str)
 void replace_var(char *str, char *var, char *value, unsigned int max)
 {
 	char *pos;
-	unsigned int lenvar = strlen(var);
-	unsigned int lenval = strlen(value);
+	size_t lenvar = strlen(var);
+	size_t lenval = strlen(value);
 
-	while((pos = strstr(str, var))) {
+	while ((pos = strstr(str, var))) {
 		/* Make room */
 		if (strlen(str) + lenval - lenvar >= max)
 			return;
@@ -140,16 +145,16 @@ char *log_build_filename(log_t *logdata, const char *destination)
 	char *dest = bip_strdup(destination);
 
 	strtolower(dest);
-	logfile = (char *)bip_malloc(MAX_PATH_LEN + 1);
+	logfile = (char *)bip_malloc((size_t)MAX_PATH_LEN + 1);
 
 	time(&s);
 	now = localtime(&s);
-	strftime(year, 5, "%Y", now);
-	strftime(day, 3, "%d", now);
-	strftime(month, 3, "%m", now);
-	strftime(hour, 3, "%H", now);
-	snprintf(logfile, MAX_PATH_LEN, "%s/%s", conf_log_root,
-			conf_log_format);
+	strftime(year, (size_t)5, "%Y", now);
+	strftime(day, (size_t)3, "%d", now);
+	strftime(month, (size_t)3, "%m", now);
+	strftime(hour, (size_t)3, "%H", now);
+	snprintf(logfile, (size_t)MAX_PATH_LEN, "%s/%s", conf_log_root,
+		 conf_log_format);
 	replace_var(logfile, "%u", logdata->user->name, MAX_PATH_LEN);
 	replace_var(logfile, "%n", logdata->network, MAX_PATH_LEN);
 	replace_var(logfile, "%c", dest, MAX_PATH_LEN);
@@ -186,6 +191,7 @@ void log_updatelast(logfile_t *lf)
 static void log_reset(logstore_t *store)
 {
 	logfile_t *olf;
+	long ftell_r;
 
 	store->skip_advance = 0;
 
@@ -195,26 +201,31 @@ static void log_reset(logstore_t *store)
 		return;
 	}
 
-	while ((olf = list_get_first(&store->file_group)) &&
-			olf != list_get_last(&store->file_group)) {
+	while ((olf = list_get_first(&store->file_group))
+	       && olf != list_get_last(&store->file_group)) {
 		logfile_free(olf);
 		list_remove_first(&store->file_group);
 	}
 
-	assert(olf);
-	assert(olf->file);
+	if (!olf || !olf->file)
+		return;
 
 	list_it_init_last(&store->file_group, &store->file_it);
 
-	fseek(olf->file, 0, SEEK_END);
-	olf->len = ftell(olf->file);
-	store->file_offset = olf->len;
+	fseek(olf->file, (long)0, SEEK_END);
+	ftell_r = ftell(olf->file);
+	if (ftell_r < 0) {
+		mylog(LOG_ERROR, "log_reset: ftell error %s", strerror(errno));
+		return;
+	}
+	store->file_offset = ftell_r;
+	olf->len = (size_t)ftell_r;
 }
 
 void log_reinit(logstore_t *store)
 {
-	mylog(LOG_ERROR, "%s is inconsistant, droping backlog info",
-			store->name);
+	mylog(LOG_ERROR, "%s is inconsistent, droping backlog info",
+	      store->name);
 	log_reset(store);
 }
 
@@ -242,10 +253,10 @@ static int log_has_file(log_t *logdata, const char *fname)
 	logstore_t *store;
 
 	for (hash_it_init(&logdata->logfgs, &hi); hash_it_item(&hi);
-			hash_it_next(&hi)) {
+	     hash_it_next(&hi)) {
 		store = hash_it_item(&hi);
 		for (list_it_init(&store->file_group, &li); list_it_item(&li);
-				list_it_next(&li)) {
+		     list_it_next(&li)) {
 			logfile_t *lf = list_it_item(&li);
 			if (strcmp(fname, lf->filename) == 0)
 				return 1;
@@ -255,13 +266,14 @@ static int log_has_file(log_t *logdata, const char *fname)
 }
 
 static int log_add_file(log_t *logdata, const char *destination,
-		const char *filename)
+			const char *filename)
 {
 	FILE *f;
 	logstore_t *store;
 	char *uniq_fname;
 	char *canonical_fname = NULL;
 	logfile_t *lf = NULL;
+	long ftell_r;
 
 	if (logdata->log_to_file) {
 		if (log_has_file(logdata, filename)) {
@@ -274,15 +286,15 @@ static int log_add_file(log_t *logdata, const char *destination,
 		f = fopen(uniq_fname, "a+");
 		if (!f) {
 			mylog(LOG_ERROR, "fopen(%s) %s", uniq_fname,
-					strerror(errno));
+			      strerror(errno));
 			free(uniq_fname);
 			free(canonical_fname);
 			return 0;
 		}
 
-		if (fseek(f, 0, SEEK_END) == -1) {
+		if (fseek(f, (long)0, SEEK_END) == -1) {
 			mylog(LOG_ERROR, "fseek(%s) %s", uniq_fname,
-					strerror(errno));
+			      strerror(errno));
 			free(uniq_fname);
 			free(canonical_fname);
 			fclose(f);
@@ -291,7 +303,16 @@ static int log_add_file(log_t *logdata, const char *destination,
 
 		lf = bip_malloc(sizeof(logfile_t));
 		lf->file = f;
-		lf->len = ftell(f);
+		ftell_r = ftell(f);
+		lf->len = (size_t)ftell_r;
+		if (ftell_r < 0) {
+			mylog(LOG_ERROR, "log_add_file: ftell error %s",
+			      strerror(errno));
+			free(uniq_fname);
+			free(canonical_fname);
+			fclose(f);
+			return 0;
+		}
 		lf->filename = uniq_fname;
 		lf->canonical_filename = canonical_fname;
 		log_updatelast(lf);
@@ -299,12 +320,13 @@ static int log_add_file(log_t *logdata, const char *destination,
 
 	store = hash_get(&logdata->logfgs, destination);
 	if (!store) {
-		store = bip_calloc(sizeof(logstore_t), 1);
+		store = bip_calloc(sizeof(logstore_t), (size_t)1);
 		list_init(&store->file_group, NULL);
 		store->name = bip_strdup(destination);
 		store->skip_advance = 0;
+		// should be safe to cast as lf->len comes from ftell()
 		if (lf)
-			store->file_offset = lf->len;
+			store->file_offset = (long)lf->len;
 		hash_insert(&logdata->logfgs, destination, store);
 	}
 
@@ -355,17 +377,17 @@ logstore_t *log_find_file(log_t *logdata, const char *destination)
 			if (!filename)
 				return NULL;
 			mylog(LOG_DEBUG, "Creating new logfile for %s: %s",
-					destination, filename);
+			      destination, filename);
 		}
 		if (!log_add_file(logdata, destination, filename)) {
-		    if (filename)
-			    free(filename);
+			if (filename)
+				free(filename);
 			return NULL;
 		}
 		store = hash_get(&logdata->logfgs, destination);
 		assert(store);
 		/* ok we are allocating a new store now, let's set it up for
-		* backlogging if applicable */
+		 * backlogging if applicable */
 		assert(logdata->user);
 		assert(logdata->network);
 		l = hash_get(&logdata->user->connections, logdata->network);
@@ -391,8 +413,8 @@ logstore_t *log_find_file(log_t *logdata, const char *destination)
 
 	time(&t);
 	ltime = localtime(&t);
-	if (ltime->tm_year != lf->last_log.tm_year ||
-			ltime->tm_yday != lf->last_log.tm_yday) {
+	if (ltime->tm_year != lf->last_log.tm_year
+	    || ltime->tm_yday != lf->last_log.tm_yday) {
 		logfile_t *oldlf;
 
 		/* day changed, we might want to rotate logfile */
@@ -408,7 +430,7 @@ logstore_t *log_find_file(log_t *logdata, const char *destination)
 
 		/* we do want to rotate logfiles */
 		mylog(LOG_DEBUG, "Rotating logfile for %s %s %s", destination,
-				lf->filename, filename);
+		      lf->filename, filename);
 
 		oldlf = list_get_last(&store->file_group);
 		if (!log_add_file(logdata, destination, filename)) {
@@ -421,8 +443,8 @@ logstore_t *log_find_file(log_t *logdata, const char *destination)
 			/* remove oldlf from file_group */
 			assert(list_remove_first(&store->file_group) == oldlf);
 			logfile_free(oldlf);
-			assert(list_get_first(&store->file_group) ==
-					list_get_last(&store->file_group));
+			assert(list_get_first(&store->file_group)
+			       == list_get_last(&store->file_group));
 		} else {
 			fclose(oldlf->file);
 			oldlf->file = NULL;
@@ -433,68 +455,65 @@ logstore_t *log_find_file(log_t *logdata, const char *destination)
 
 void log_join(log_t *logdata, const char *ircmask, const char *channel)
 {
-	snprintf(logdata->buffer, LOGLINE_MAXLEN,
-			"%s -!- %s has joined %s", timestamp(), ircmask,
-			channel);
+	snprintf(logdata->buffer, (size_t)LOGLINE_MAXLEN,
+		 "%s -!- %s has joined %s", timestamp(), ircmask, channel);
 	log_write(logdata, channel, logdata->buffer);
 }
 
 void log_part(log_t *logdata, const char *ircmask, const char *channel,
-		const char *message)
+	      const char *message)
 {
 	if (message)
-		snprintf(logdata->buffer, LOGLINE_MAXLEN,
-			"%s -!- %s has left %s [%s]", timestamp(), ircmask,
-			channel, message);
+		snprintf(logdata->buffer, (size_t)LOGLINE_MAXLEN,
+			 "%s -!- %s has left %s [%s]", timestamp(), ircmask,
+			 channel, message);
 	else
-		snprintf(logdata->buffer, LOGLINE_MAXLEN,
-			"%s -!- %s has left %s", timestamp(), ircmask,
-			channel);
+		snprintf(logdata->buffer, (size_t)LOGLINE_MAXLEN,
+			 "%s -!- %s has left %s", timestamp(), ircmask,
+			 channel);
 	log_write(logdata, channel, logdata->buffer);
 }
 
 void log_kick(log_t *logdata, const char *ircmask, const char *channel,
-		const char *who, const char *message)
+	      const char *who, const char *message)
 {
-	snprintf(logdata->buffer, LOGLINE_MAXLEN,
-			"%s -!- %s has been kicked by %s [%s]", timestamp(),
-			who, ircmask, message);
+	snprintf(logdata->buffer, (size_t)LOGLINE_MAXLEN,
+		 "%s -!- %s has been kicked by %s [%s]", timestamp(), who,
+		 ircmask, message);
 	log_write(logdata, channel, logdata->buffer);
 }
 
 void log_quit(log_t *logdata, const char *ircmask, const char *channel,
-		const char *message)
+	      const char *message)
 {
-	snprintf(logdata->buffer, LOGLINE_MAXLEN,
-			"%s -!- %s has quit [%s]", timestamp(), ircmask,
-			message);
+	snprintf(logdata->buffer, (size_t)LOGLINE_MAXLEN,
+		 "%s -!- %s has quit [%s]", timestamp(), ircmask, message);
 	log_write(logdata, channel, logdata->buffer);
 }
 
 void log_nick(log_t *logdata, const char *ircmask, const char *channel,
-		const char *newnick)
+	      const char *newnick)
 {
 	char *oldnick = nick_from_ircmask(ircmask);
-	logstore_t* oldstore;
-	logstore_t* newstore;
+	logstore_t *oldstore;
+	logstore_t *newstore;
 
 	if ((oldstore = hash_get(&logdata->logfgs, oldnick))) {
 		if ((newstore = hash_get(&logdata->logfgs, newnick))
-			&& oldstore != newstore) {
+		    && oldstore != newstore) {
 			log_drop(logdata, newnick);
 		}
 		hash_rename_key(&logdata->logfgs, oldnick, newnick);
 	}
 	free(oldnick);
 
-	snprintf(logdata->buffer, LOGLINE_MAXLEN,
-			"%s -!- %s is now known as %s",
-			timestamp(), ircmask, newnick);
+	snprintf(logdata->buffer, (size_t)LOGLINE_MAXLEN,
+		 "%s -!- %s is now known as %s", timestamp(), ircmask, newnick);
 	log_write(logdata, channel, logdata->buffer);
 }
 
 static void do_log_privmsg(log_t *logdata, const char *storage, int src,
-		const char *from, const char *message)
+			   const char *from, const char *message)
 {
 	char dir = '<';
 
@@ -503,10 +522,11 @@ static void do_log_privmsg(log_t *logdata, const char *storage, int src,
 	if (src)
 		dir = '>';
 
-	if (strlen(message) > 8 && ((*message == '\001' ||
-		((*message == '+' || *message == '-') &&
-			 (*(message + 1) == '\001'))) &&
-		(*(message + strlen(message) - 1) == '\001'))) {
+	if (strlen(message) > 8
+	    && ((*message == '\001'
+		 || ((*message == '+' || *message == '-')
+		     && (*(message + 1) == '\001')))
+		&& (*(message + strlen(message) - 1) == '\001'))) {
 		char *msg;
 		/* hack for freenode and the like */
 		const char *real_message = message;
@@ -514,25 +534,27 @@ static void do_log_privmsg(log_t *logdata, const char *storage, int src,
 		if (*message == '+' || *message == '-')
 			real_message++;
 
-		if (strncmp(real_message, "\001ACTION ", 8) != 0)
+		if (strncmp(real_message, "\001ACTION ", (size_t)8) != 0)
 			return;
 		msg = bip_strdup(real_message);
 		*(msg + strlen(msg) - 1) = '\0';
-		snprintf(logdata->buffer, LOGLINE_MAXLEN,
-					"%s %c * %s %s", timestamp(), dir,
-					from, msg + 8);
+		snprintf(logdata->buffer, (size_t)LOGLINE_MAXLEN,
+			 "%s %c * %s %s", timestamp(), dir, from, msg + 8);
 		free(msg);
 	} else {
-		snprintf(logdata->buffer, LOGLINE_MAXLEN,
-					"%s %c %s: %s", timestamp(), dir,
-					from, message);
+		snprintf(logdata->buffer, (size_t)LOGLINE_MAXLEN,
+			 "%s %c %s: %s", timestamp(), dir, from, message);
 	}
 	log_write(logdata, storage, logdata->buffer);
 }
 
 void log_privmsg(log_t *logdata, const char *ircmask, const char *destination,
-		const char *message)
+		 const char *message)
 {
+// TODO resolve this issue from array_get
+// passing argument X of .... with different width due to prototype
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wtraditional-conversion"
 	if (!ischannel(*destination)) {
 		char *nick = nick_from_ircmask(ircmask);
 		do_log_privmsg(logdata, nick, 0, ircmask, message);
@@ -540,10 +562,11 @@ void log_privmsg(log_t *logdata, const char *ircmask, const char *destination,
 	} else {
 		do_log_privmsg(logdata, destination, 0, ircmask, message);
 	}
+#pragma GCC diagnostic pop
 }
 
 void log_cli_privmsg(log_t *logdata, const char *ircmask,
-		const char *destination, const char *message)
+		     const char *destination, const char *message)
 {
 	do_log_privmsg(logdata, destination, 1, ircmask, message);
 }
@@ -553,6 +576,10 @@ void log_notice(log_t *logdata, const char *ircmask, const char *destination,
 {
 	if (!ircmask)
 		ircmask = P_IRCMASK;
+// TODO resolve this issue from array_get
+// passing argument X of .... with different width due to prototype
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wtraditional-conversion"
 	if (!ischannel(*destination)) {
 		char *nick = nick_from_ircmask(ircmask);
 		do_log_privmsg(logdata, nick, 0, ircmask, message);
@@ -560,33 +587,33 @@ void log_notice(log_t *logdata, const char *ircmask, const char *destination,
 	} else {
 		do_log_privmsg(logdata, destination, 0, ircmask, message);
 	}
+#pragma GCC diagnostic pop
 }
 
 void log_cli_notice(log_t *logdata, const char *ircmask,
-		const char *destination, const char *message)
+		    const char *destination, const char *message)
 {
 	do_log_privmsg(logdata, destination, 1, ircmask, message);
 }
 
 void log_topic(log_t *logdata, const char *ircmask, const char *channel,
-		const char *message)
+	       const char *message)
 {
-	snprintf(logdata->buffer, LOGLINE_MAXLEN,
-			"%s -!- %s changed topic of %s to: %s", timestamp(),
-			ircmask, channel, message);
+	snprintf(logdata->buffer, (size_t)LOGLINE_MAXLEN,
+		 "%s -!- %s changed topic of %s to: %s", timestamp(), ircmask,
+		 channel, message);
 	log_write(logdata, channel, logdata->buffer);
 }
 
 void log_init_topic(log_t *logdata, const char *channel, const char *message)
 {
-	snprintf(logdata->buffer, LOGLINE_MAXLEN,
-			"%s -!- Topic for %s: %s", timestamp(), channel,
-			message);
+	snprintf(logdata->buffer, (size_t)LOGLINE_MAXLEN,
+		 "%s -!- Topic for %s: %s", timestamp(), channel, message);
 	log_write(logdata, channel, logdata->buffer);
 }
 
 void log_init_topic_time(log_t *logdata, const char *channel, const char *who,
-		const char *when)
+			 const char *when)
 {
 	struct tm *time;
 	char *timestr;
@@ -594,40 +621,40 @@ void log_init_topic_time(log_t *logdata, const char *channel, const char *who,
 
 	seconds = atoi(when);
 	time = localtime(&seconds);
-	timestr = (char *)bip_malloc(50 + 1);
+	timestr = (char *)bip_malloc((size_t)50 + 1);
 	timestr[0] = '\0';
 	if (time)
-		strftime(timestr, 50, "%A %d %B %Y, %H:%M:%S", time);
+		strftime(timestr, (size_t)50, "%A %d %B %Y, %H:%M:%S", time);
 	timestr[50] = '\0';
 
-	snprintf(logdata->buffer, LOGLINE_MAXLEN,
-			"%s -!- Topic set by %s [%s]", timestamp(), who,
-			timestr);
+	snprintf(logdata->buffer, (size_t)LOGLINE_MAXLEN,
+		 "%s -!- Topic set by %s [%s]", timestamp(), who, timestr);
 	free(timestr);
 	log_write(logdata, channel, logdata->buffer);
 }
 
 void log_mode(log_t *logdata, const char *ircmask, const char *channel,
-		const char *modes, array_t *mode_args)
+	      const char *modes, array_t *mode_args)
 {
 	int i;
-	char *tmpbuf = bip_malloc(LOGLINE_MAXLEN + 1);
-	char *tmpbuf2 = bip_malloc(LOGLINE_MAXLEN + 1);
+	char *tmpbuf = bip_malloc((size_t)LOGLINE_MAXLEN + 1);
+	char *tmpbuf2 = bip_malloc((size_t)LOGLINE_MAXLEN + 1);
 	char *tmp;
 
-	snprintf(tmpbuf, LOGLINE_MAXLEN, "%s -!- mode/%s [%s", timestamp(),
-			channel, modes);
+	snprintf(tmpbuf, (size_t)LOGLINE_MAXLEN, "%s -!- mode/%s [%s",
+		 timestamp(), channel, modes);
 	if (mode_args) {
 		for (i = 0; i < array_count(mode_args); i++) {
-			snprintf(tmpbuf2, LOGLINE_MAXLEN, "%s %s", tmpbuf,
-					(char *)array_get(mode_args, i));
+			snprintf(tmpbuf2, (size_t)LOGLINE_MAXLEN, "%s %s",
+				 tmpbuf, (const char *)array_get(mode_args, i));
 			tmp = tmpbuf;
 			tmpbuf = tmpbuf2;
 			tmpbuf2 = tmp;
 		}
 	}
 
-	snprintf(logdata->buffer, LOGLINE_MAXLEN, "%s] by %s", tmpbuf, ircmask);
+	snprintf(logdata->buffer, (size_t)LOGLINE_MAXLEN, "%s] by %s", tmpbuf,
+		 ircmask);
 	log_write(logdata, channel, logdata->buffer);
 
 	free(tmpbuf);
@@ -637,10 +664,12 @@ void log_mode(log_t *logdata, const char *ircmask, const char *channel,
 void log_disconnected(log_t *logdata)
 {
 	hash_iterator_t hi;
-	snprintf(logdata->buffer, LOGLINE_MAXLEN, "%s -!- Disconnected"
-			" from server...", timestamp());
+	snprintf(logdata->buffer, (size_t)LOGLINE_MAXLEN,
+		 "%s -!- Disconnected"
+		 " from server...",
+		 timestamp());
 	for (hash_it_init(&logdata->logfgs, &hi); hash_it_item(&hi);
-			hash_it_next(&hi))
+	     hash_it_next(&hi))
 		log_write(logdata, hash_it_key(&hi), logdata->buffer);
 }
 
@@ -649,8 +678,8 @@ void log_ping_timeout(log_t *logdata)
 	list_t *l = log_backlogs(logdata);
 	char *s;
 
-	snprintf(logdata->buffer, LOGLINE_MAXLEN,
-			"%s -!- Ping timeout with server...", timestamp());
+	snprintf(logdata->buffer, (size_t)LOGLINE_MAXLEN,
+		 "%s -!- Ping timeout with server...", timestamp());
 	while ((s = list_remove_first(l))) {
 		log_write(logdata, s, logdata->buffer);
 		free(s);
@@ -663,10 +692,12 @@ void log_connected(log_t *logdata)
 {
 	hash_iterator_t hi;
 
-	snprintf(logdata->buffer, LOGLINE_MAXLEN, "%s -!- Connected to"
-			" server...", timestamp());
+	snprintf(logdata->buffer, (size_t)LOGLINE_MAXLEN,
+		 "%s -!- Connected to"
+		 " server...",
+		 timestamp());
 	for (hash_it_init(&logdata->logfgs, &hi); hash_it_item(&hi);
-			hash_it_next(&hi)) {
+	     hash_it_next(&hi)) {
 		log_write(logdata, hash_it_key(&hi), logdata->buffer);
 	}
 }
@@ -707,12 +738,17 @@ void log_reset_all(log_t *logdata)
 	list_init(&drop, NULL);
 
 	for (hash_it_init(&logdata->logfgs, &hi); hash_it_item(&hi);
-			hash_it_next(&hi)) {
+	     hash_it_next(&hi)) {
 		store = hash_it_item(&hi);
+// TODO resolve this issue from array_get
+// passing argument X of .... with different width due to prototype
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wtraditional-conversion"
 		if (ischannel(*hash_it_key(&hi)))
 			log_reset(store);
 		else
 			list_add_last(&drop, bip_strdup(hash_it_key(&hi)));
+#pragma GCC diagnostic pop
 	}
 
 	char *name;
@@ -729,8 +765,13 @@ void log_reset_store(log_t *log, const char *storename)
 	store = hash_get(&log->logfgs, storename);
 	if (store) {
 		log_reset(store);
+// TODO resolve this issue from array_get
+// passing argument X of .... with different width due to prototype
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wtraditional-conversion"
 		if (!ischannel(*storename))
 			log_drop(log, storename);
+#pragma GCC diagnostic pop
 	}
 }
 
@@ -750,11 +791,11 @@ void log_client_connected(log_t *logdata)
 	logdata->connected = 1;
 }
 
-void log_advance_backlogs(log_t* ld, logstore_t *store)
+void log_advance_backlogs(log_t *ld, logstore_t *store)
 {
 	int c;
 
-	//mylog(LOG_ERROR, "LOG: log_advance_backlogs %s", store->name);
+	// mylog(LOG_ERROR, "LOG: log_advance_backlogs %s", store->name);
 	if (!store->track_backlog)
 		return;
 
@@ -768,18 +809,20 @@ void log_advance_backlogs(log_t* ld, logstore_t *store)
 
 	logfile_t *lf;
 	while ((lf = list_it_item(&store->file_it))) {
-		//mylog(LOG_ERROR, "LOG: %s %d", lf->filename, store->file_offset);
+		// mylog(LOG_ERROR, "LOG: %s %d", lf->filename,
+		// store->file_offset);
 		if (!lf->file) {
 			lf->file = fopen(lf->filename, "r");
 			if (!lf->file) {
 				mylog(LOG_ERROR, "Can't open %s for reading",
-						lf->filename);
+				      lf->filename);
 				log_reinit(store);
 				return;
 			}
 		}
 		if (fseek(lf->file, store->file_offset, SEEK_SET)) {
-			//mylog(LOG_ERROR, "LOG: fseek FAIL %s %d", lf->filename, store->file_offset);
+			// mylog(LOG_ERROR, "LOG: fseek FAIL %s %d",
+			// lf->filename, store->file_offset);
 			log_reinit(store);
 			return;
 		}
@@ -787,16 +830,20 @@ void log_advance_backlogs(log_t* ld, logstore_t *store)
 		while ((c = fgetc(lf->file)) != EOF) {
 			store->file_offset++;
 			if (c == '\n') {
-				//mylog(LOG_ERROR, "LOG: %s advanced at %d", lf->filename, store->file_offset);
+				// mylog(LOG_ERROR, "LOG: %s advanced at %d",
+				// lf->filename, store->file_offset);
 				return;
 			}
 		}
-		//mylog(LOG_ERROR, "LOG: %s advanced at %d", lf->filename, store->file_offset);
+		// mylog(LOG_ERROR, "LOG: %s advanced at %d", lf->filename,
+		// store->file_offset);
 		if (lf == list_get_last(&store->file_group)) {
-			//mylog(LOG_ERROR, "LOG: %s is last of file_group", lf->filename, store->file_offset);
+			// mylog(LOG_ERROR, "LOG: %s is last of file_group",
+			// lf->filename, store->file_offset);
 			return;
 		}
-		//mylog(LOG_ERROR, "LOG: next file %s %d", lf->filename, store->file_offset);
+		// mylog(LOG_ERROR, "LOG: next file %s %d", lf->filename,
+		// store->file_offset);
 		fclose(lf->file);
 		lf->file = NULL;
 		list_it_next(&store->file_it);
@@ -822,7 +869,9 @@ int log_has_backlog(log_t *logdata, const char *destination)
 	if (lf != list_get_last(&store->file_group))
 		return 1;
 
-	return store->file_offset != lf->len;
+	// should be safe to cast to unsigned as we check ftell
+	// when setting file_offset and only ++ since then
+	return (size_t)store->file_offset != lf->len;
 }
 
 /*
@@ -834,7 +883,7 @@ chan:
 09-01-2009 14:16:21 < nohar!~nohar@haruka.t1r.net: chantestrepl
 */
 char *log_beautify(log_t *logdata, const char *buf, const char *storename,
-		const char *dest)
+		   const char *dest)
 {
 	int action = 0;
 	char *p;
@@ -850,17 +899,21 @@ char *log_beautify(log_t *logdata, const char *buf, const char *storename,
 	assert(buf);
 
 	p = strchr(buf, ' ');
+	// buf...p
 	if (!p || !p[0] || !p[1])
 		return _log_wrap(dest, buf);
 	p++;
+	// buf...p
 	sots = logdata->user->backlog_timestamp == BLTSDateTime ? buf : p;
+	// buf...sots=p OR sots=buf...p
 	p = strchr(p, ' ');
+	// buf...sots...p OR sots=buf...p
 	if (!p || !p[0] || !p[1])
 		return _log_wrap(dest, buf);
-	lots = p - sots;
+	lots = (size_t)(p - sots);
 	p++;
 
-	if (strncmp(p, "-!-", 3) == 0) {
+	if (strncmp(p, "-!-", (size_t)3) == 0) {
 		if (logdata->user->bl_msg_only)
 			return NULL;
 		else
@@ -890,27 +943,33 @@ char *log_beautify(log_t *logdata, const char *buf, const char *storename,
 		p++;
 	if (!p[0])
 		return _log_wrap(dest, buf);
-	lon = p - son;
+	lon = (size_t)(p - son);
 
 	p = strchr(p, ' ');
 	if (!p || !p[0] || !p[1])
 		return _log_wrap(dest, buf);
 	p++;
 
+// TODO resolve this issue from array_get
+// passing argument X of .... with different width due to prototype
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wtraditional-conversion"
 	if (out && !ischannel(*dest)) {
 		son = storename;
 		lon = strlen(storename);
 	}
+#pragma GCC diagnostic pop
 
 	som = p;
 	lom = strlen(p);
 	if (lom == 0)
 		return _log_wrap(dest, buf);
 
-	p = ret = (char *)bip_malloc(
-		1 + lon + strlen(LAMESTRING) + strlen(dest) + 2 + lots + 2 +
-		lom + 3 + action * (2 + strlen("ACTION ")) +
-		out * strlen(PMSG_ARROW));
+	// action and out are 0 or 1, safe to cast
+	p = ret = (char *)bip_malloc(1 + lon + strlen(LAMESTRING) + strlen(dest)
+				     + 2 + lots + 2 + lom + 3
+				     + (size_t)action * (2 + strlen("ACTION "))
+				     + (size_t)out * strlen(PMSG_ARROW));
 
 	*p++ = ':';
 
@@ -922,8 +981,8 @@ char *log_beautify(log_t *logdata, const char *buf, const char *storename,
 
 	memcpy(p, dest, strlen(dest));
 	p += strlen(dest);
-	//memcpy(p, sod, lod);
-	//p += lod;
+	// memcpy(p, sod, lod);
+	// p += lod;
 
 	*p++ = ' ';
 	*p++ = ':';
@@ -933,10 +992,15 @@ char *log_beautify(log_t *logdata, const char *buf, const char *storename,
 		strcpy(p, "ACTION ");
 		p += strlen("ACTION ");
 	}
+// TODO resolve this issue from array_get
+// passing argument X of .... with different width due to prototype
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wtraditional-conversion"
 	if (out && !ischannel(*dest)) {
 		strcpy(p, PMSG_ARROW);
 		p += strlen(PMSG_ARROW);
 	}
+#pragma GCC diagnostic pop
 	if (logdata->user->backlog_timestamp != BLTSNone) {
 		memcpy(p, sots, lots);
 		p += lots;
@@ -972,20 +1036,20 @@ static time_t compute_time(const char *buf)
 }
 
 static int log_backread_file(log_t *log, logstore_t *store, logfile_t *lf,
-		list_t *res, const char *dest, time_t start)
+			     list_t *res, const char *dest, time_t start)
 {
 	char *buf, *logbr;
 	int close = 0;
 
-	//mylog(LOG_ERROR, "log_backread_file store:", store->name);
+	// mylog(LOG_ERROR, "log_backread_file store:", store->name);
 
 	if (!lf->file) {
 		lf->file = fopen(lf->filename, "r");
 		if (!lf->file) {
 			mylog(LOG_ERROR, "Can't open %s for reading",
-					lf->filename);
+			      lf->filename);
 			list_add_last(res, _log_wrap("Error opening logfile",
-					store->name));
+						     store->name));
 			return 0;
 		}
 
@@ -999,31 +1063,36 @@ static int log_backread_file(log_t *log, logstore_t *store, logfile_t *lf,
 		*/
 		if (fseek(lf->file, store->file_offset, SEEK_SET)) {
 			mylog(LOG_ERROR, "Can't seek in %s", lf->filename);
-			list_add_last(res, _log_wrap(store->name,
+			list_add_last(res,
+				      _log_wrap(store->name,
 						"Error seeking in logfile"));
 			return 0;
 		}
 	} else {
-		//mylog(LOG_ERROR, "bread Seeking %s to %d", lf->filename, 0);
-		if (fseek(lf->file, 0, SEEK_SET)) {
+		// mylog(LOG_ERROR, "bread Seeking %s to %d", lf->filename, 0);
+		if (fseek(lf->file, (long)0, SEEK_SET)) {
 			mylog(LOG_ERROR, "Can't seek in %s", lf->filename);
-			list_add_last(res, _log_wrap(store->name,
+			list_add_last(res,
+				      _log_wrap(store->name,
 						"Error seeking in logfile"));
 			return 0;
 		}
 	}
 
-	buf = bip_malloc(LOGLINE_MAXLEN + 1);
-	for(;;) {
+	buf = bip_malloc((size_t)LOGLINE_MAXLEN + 1);
+	for (;;) {
 		if (!fgets(buf, LOGLINE_MAXLEN, lf->file)) {
 			if (ferror(lf->file)) {
 				list_add_last(res, _log_wrap("Error reading "
-						"logfile", store->name));
+							     "logfile",
+							     store->name));
 			}
 			/* error or oef */
 			break;
 		}
-		int slen = strlen(buf);
+		size_t slen = strlen(buf);
+		if (slen == 0)
+			break; // should not happen, per previous fgets block
 		if (buf[slen - 1] == '\n')
 			buf[slen - 1] = 0;
 		if (slen >= 2 && buf[slen] == '\r')
@@ -1036,8 +1105,8 @@ static int log_backread_file(log_t *log, logstore_t *store, logfile_t *lf,
 			/* parse error, don't backlog */
 			if (linetime == (time_t)-1) {
 				list_add_last(res, _log_wrap("Error in "
-							"timestamp in %s",
-							store->name));
+							     "timestamp in %s",
+							     store->name));
 				continue;
 			}
 			/* too old line, don't backlog */
@@ -1047,14 +1116,13 @@ static int log_backread_file(log_t *log, logstore_t *store, logfile_t *lf,
 		logbr = log_beautify(log, buf, store->name, dest);
 		if (logbr)
 			list_add_last(res, logbr);
-
 	}
 	if (close) {
 		fclose(lf->file);
 		lf->file = NULL;
 	}
 	free(buf);
-	//mylog(LOG_ERROR, "end of log_backread_file store: %s", store->name);
+	// mylog(LOG_ERROR, "end of log_backread_file store: %s", store->name);
 	return 1;
 }
 
@@ -1077,7 +1145,7 @@ static list_t *log_backread(log_t *log, const char *storename, const char *dest)
 		ret = list_new(NULL);
 
 		for (list_it_init(store->memlog, &li); list_it_item(&li);
-				list_it_next(&li))
+		     list_it_next(&li))
 			list_add_last(ret, bip_strdup(list_it_item(&li)));
 		return ret;
 	}
@@ -1091,11 +1159,10 @@ static list_t *log_backread(log_t *log, const char *storename, const char *dest)
 	logfile_t *logf;
 
 	ret = list_new(NULL);
-	for (file_it = store->file_it;
-			(logf = list_it_item(&file_it));
-			list_it_next(&file_it)) {
+	for (file_it = store->file_it; (logf = list_it_item(&file_it));
+	     list_it_next(&file_it)) {
 		if (!log_backread_file(log, store, logf, ret, dest,
-					(time_t)0)) {
+				       (time_t)0)) {
 			log_reinit(store);
 			return ret;
 		}
@@ -1106,13 +1173,21 @@ static list_t *log_backread(log_t *log, const char *storename, const char *dest)
 static char *_log_wrap(const char *dest, const char *line)
 {
 	char *buf;
-	size_t count;
+	int count;
 
-	buf = bip_malloc(LOGLINE_MAXLEN + 1);
-	count = snprintf(buf, LOGLINE_MAXLEN + 1,
-			":" P_IRCMASK " PRIVMSG %s :%s\r\n", dest, line);
+	buf = bip_malloc((size_t)LOGLINE_MAXLEN + 1);
+	count = snprintf(buf, (size_t)LOGLINE_MAXLEN + 1,
+			 ":" P_IRCMASK " PRIVMSG %s :%s\r\n", dest, line);
+	if (count < 0) {
+		mylog(LOG_ERROR, "_log_wrap: error on snprintf: %s",
+		      strerror(errno));
+		buf[LOGLINE_MAXLEN - 2] = '\r';
+		buf[LOGLINE_MAXLEN - 1] = '\n';
+		buf[LOGLINE_MAXLEN] = 0;
+		return buf;
+	}
 	if (count >= LOGLINE_MAXLEN + 1) {
-		mylog(LOG_DEBUG, "line too long");
+		mylog(LOG_WARN, "_log_wrap: line too long");
 		buf[LOGLINE_MAXLEN - 2] = '\r';
 		buf[LOGLINE_MAXLEN - 1] = '\n';
 		buf[LOGLINE_MAXLEN] = 0;
@@ -1120,19 +1195,19 @@ static char *_log_wrap(const char *dest, const char *line)
 	return buf;
 }
 
-static int _log_write(log_t *logdata, logstore_t *store,
-		const char *destination, const char *str)
+static size_t _log_write(log_t *logdata, logstore_t *store,
+			 const char *destination, const char *str)
 {
 	size_t nbwrite;
 	size_t len;
 	static char tmpstr[LOGLINE_MAXLEN + 1];
 
-	strncpy(tmpstr, str, LOGLINE_MAXLEN);
+	strncpy(tmpstr, str, (size_t)LOGLINE_MAXLEN);
 	tmpstr[LOGLINE_MAXLEN] = 0;
 
 	if (store->memlog) {
-		char *r = log_beautify(logdata, tmpstr, store->name,
-				destination);
+		char *r =
+			log_beautify(logdata, tmpstr, store->name, destination);
 		if (r != NULL) {
 			list_add_last(store->memlog, r);
 			if (store->memc == logdata->user->backlog_lines)
@@ -1149,7 +1224,7 @@ static int _log_write(log_t *logdata, logstore_t *store,
 
 	len = strlen(tmpstr);
 	nbwrite = fwrite(tmpstr, sizeof(char), len, lf->file);
-	nbwrite += fwrite("\n", sizeof(char), 1, lf->file);
+	nbwrite += fwrite("\n", sizeof(char), (size_t)1, lf->file);
 	log_updatelast(lf);
 	if (nbwrite != len + 1)
 		mylog(LOG_ERROR, "Error writing to %s logfile", lf->filename);
@@ -1162,13 +1237,16 @@ static int _log_write(log_t *logdata, logstore_t *store,
 void log_write(log_t *logdata, const char *destination, const char *str)
 {
 	logstore_t *store = log_find_file(logdata, destination);
+	size_t written;
 
 	if (!store) {
 		mylog(LOG_ERROR, "Unable to find/create logfile for '%s'",
-				destination);
+		      destination);
 		return;
 	}
-	_log_write(logdata, store, destination, str);
+	written = _log_write(logdata, store, destination, str);
+	if (written <= 0)
+		mylog(LOG_WARN, "log_write to '%s' failed", destination);
 }
 
 static list_t *log_all_logs = NULL;
@@ -1181,15 +1259,15 @@ void log_flush_all(void)
 
 	fflush(conf_global_log_file);
 	for (list_it_init(log_all_logs, &li); list_it_item(&li);
-			list_it_next(&li)) {
+	     list_it_next(&li)) {
 		log_t *log = list_it_item(&li);
 		hash_iterator_t hi;
 		for (hash_it_init(&log->logfgs, &hi); hash_it_item(&hi);
-				hash_it_next(&hi)) {
+		     hash_it_next(&hi)) {
 			logstore_t *store = hash_it_item(&hi);
 			list_iterator_t lj;
 			for (list_it_init(&store->file_group, &lj);
-					list_it_item(&lj); list_it_next(&lj)) {
+			     list_it_item(&lj); list_it_next(&lj)) {
 				logfile_t *lf = list_it_item(&lj);
 				if (lf->file)
 					fflush(lf->file);
@@ -1202,11 +1280,11 @@ log_t *log_new(struct bipuser *user, const char *network)
 {
 	log_t *logdata;
 
-	logdata = (log_t *)bip_calloc(sizeof(log_t), 1);
+	logdata = (log_t *)bip_calloc(sizeof(log_t), (size_t)1);
 	logdata->user = user;
 	logdata->network = bip_strdup(network);
 	hash_init(&logdata->logfgs, HASH_NOCASE);
-	logdata->buffer = (char *)bip_malloc(LOGLINE_MAXLEN + 1);
+	logdata->buffer = (char *)bip_malloc((size_t)LOGLINE_MAXLEN + 1);
 	logdata->buffer[LOGLINE_MAXLEN - 1] = 0; // debug
 	logdata->buffer[LOGLINE_MAXLEN] = 0;
 	logdata->connected = 0;
@@ -1228,7 +1306,7 @@ void log_free(log_t *log)
 	free(log->buffer);
 
 	for (hash_it_init(&log->logfgs, &it); (store = hash_it_item(&it));
-			hash_it_next(&it)) {
+	     hash_it_next(&it)) {
 		log_reset(store);
 		log_store_free(store);
 	}
@@ -1245,13 +1323,14 @@ array_t *str_split(const char *str, const char *splt)
 {
 	const char *p = str;
 	const char *start = str;
-	int len;
+	size_t len;
 	char *extracted;
-	array_t *array = array_new();;
+	array_t *array = array_new();
+	;
 
 	do {
 		if (!*p || strchr(splt, *p)) {
-			len = p - start;
+			len = (size_t)(p - start);
 			extracted = bip_malloc(len + 1);
 			memcpy(extracted, start, len);
 			extracted[len] = 0;
@@ -1268,7 +1347,7 @@ array_t *str_split(const char *str, const char *splt)
 
 /* 26-12-2008 08:54:39 */
 int log_parse_date(char *strdate, int *year, int *month, int *mday, int *hour,
-		int *min, int *sec)
+		   int *min, int *sec)
 {
 	array_t *fields;
 	char *sptr;
@@ -1285,8 +1364,7 @@ int log_parse_date(char *strdate, int *year, int *month, int *mday, int *hour,
 		ret = 1;
 	}
 	int i;
-	array_each(fields, i, sptr)
-		free(sptr);
+	array_each(fields, i, sptr) free(sptr);
 
 	array_free(fields);
 	return ret;
@@ -1295,7 +1373,7 @@ int log_parse_date(char *strdate, int *year, int *month, int *mday, int *hour,
 void logstore_get_file_at(logstore_t *store, time_t at, list_iterator_t *li)
 {
 	for (list_it_init(&store->file_group, li); list_it_item(li);
-			list_it_next(li)) {
+	     list_it_next(li)) {
 		logfile_t *lf = list_it_item(li);
 
 		if (mktime(&lf->last_log) > at)
@@ -1304,7 +1382,7 @@ void logstore_get_file_at(logstore_t *store, time_t at, list_iterator_t *li)
 }
 
 static list_t *log_backread_hours(log_t *log, const char *storename,
-		const char *dest, int hours)
+				  const char *dest, int hours)
 {
 	time_t blstarttime;
 	struct timeval tv;
@@ -1324,10 +1402,9 @@ static list_t *log_backread_hours(log_t *log, const char *storename,
 
 	ret = list_new(NULL);
 	for (logstore_get_file_at(store, blstarttime, &file_it);
-			(logf = list_it_item(&file_it));
-			list_it_next(&file_it)) {
+	     (logf = list_it_item(&file_it)); list_it_next(&file_it)) {
 		if (!log_backread_file(log, store, logf, ret, dest,
-					blstarttime)) {
+				       blstarttime)) {
 			log_reinit(store);
 			return ret;
 		}
@@ -1336,17 +1413,22 @@ static list_t *log_backread_hours(log_t *log, const char *storename,
 }
 
 list_t *backlog_lines(log_t *log, const char *bl, const char *cli_nick,
-		int hours)
+		      int hours)
 {
 	list_t *ret;
 	struct line l;
 	const char *dest;
 
 	ret = NULL;
+// TODO resolve this issue from array_get
+// passing argument X of .... with different width due to prototype
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wtraditional-conversion"
 	if (ischannel(*bl))
 		dest = bl;
 	else
 		dest = cli_nick;
+#pragma GCC diagnostic pop
 
 	if (log_has_backlog(log, bl) || hours) {
 		if (hours == 0)
@@ -1365,7 +1447,7 @@ list_t *backlog_lines(log_t *log, const char *bl, const char *cli_nick,
 				irc_line_init(&l);
 				l.origin = P_IRCMASK;
 				if (dest == cli_nick)
-					l.origin = (char *)bl;
+					l.origin = bip_strdup(bl);
 				_irc_line_append(&l, "PRIVMSG");
 				_irc_line_append(&l, dest);
 				_irc_line_append(&l, "End of backlog");
@@ -1376,4 +1458,3 @@ list_t *backlog_lines(log_t *log, const char *bl, const char *cli_nick,
 	}
 	return ret;
 }
-
