@@ -2,7 +2,8 @@
  * $Id: util.c,v 1.60 2005/04/12 19:34:35 nohar Exp $
  *
  * This file is part of the bip project
- * Copyright (C) 2004 2005 Arnaud Cornet and Loïc Gomez
+ * Copyright (C) 2004,2005 Arnaud Cornet
+ * Copyright (C) 2004,2005,2022 Loïc Gomez
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,14 +29,17 @@
 
 extern int conf_log_level;
 extern int conf_log_system;
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wstrict-prototypes"
 extern int errno;
+#pragma GCC diagnostic pop
 extern FILE *conf_global_log_file;
 
 void memory_fatal(void)
 {
 	fflush(conf_global_log_file);
 #define OOMMSG "Out of memory.\n"
-	fwrite(OOMMSG, 1, strlen(OOMMSG), conf_global_log_file);
+	fwrite(OOMMSG, (size_t)1, strlen(OOMMSG), conf_global_log_file);
 #undef OOMMSG
 	fflush(conf_global_log_file);
 	exit(28);
@@ -73,6 +77,17 @@ void *bip_realloc(void *ptr, size_t size)
 	return r;
 }
 
+void bip_cfree(const void *ptr)
+{
+	if (!ptr)
+		return;
+// there's no other way to free a const pointer
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wcast-qual"
+	free((void *)ptr);
+#pragma GCC diagnostic pop
+}
+
 char *bip_strdup(const char *str)
 {
 	char *r = strdup(str);
@@ -81,6 +96,98 @@ char *bip_strdup(const char *str)
 	return r;
 }
 
+char *bip_strcat_fit(size_t *remaining, char *str, const char *str2)
+{
+	char *res;
+
+	if (!remaining || !str || !str2) {
+		mylog(LOG_DEBUGVERB, "bip_strcat_fit: got NULL pointer");
+		return NULL;
+	}
+
+	res = memccpy(str, str2, '\0', *remaining);
+	if (!res) {
+		mylog(LOG_DEBUGTOOMUCH,
+		      "bip_strcat_fit: memccpy() failed, remaining %lu",
+		      *remaining);
+		return NULL;
+	}
+
+	res--;
+	if (res < str) {
+		mylog(LOG_DEBUG, "bip_strcat_fit: memccpy res < str");
+		return NULL;
+	}
+	(*remaining) -= (size_t)(res - str);
+	return res;
+}
+
+#define STRCATF_BUF_MAXLEN 1024
+char *bip_strcatf_fit(size_t *remaining, char *str, const char *fmt, ...)
+{
+	va_list ap;
+	char str2[STRCATF_BUF_MAXLEN + 1];
+	int written;
+	char *res = NULL;
+
+	if (!remaining || !str || !fmt) {
+		mylog(LOG_DEBUGVERB, "bip_strcatf_fit: NULL pointer");
+		return NULL;
+	}
+
+	if (*remaining > STRCATF_BUF_MAXLEN) {
+		mylog(LOG_ERROR,
+		      "bip_strcatf_fit: remaining "
+		      "is over STRCATF_BUF_MAXLEN");
+	}
+
+	va_start(ap, fmt);
+	str2[*remaining] = '\0';
+	written = vsnprintf(str2, *remaining, fmt, ap);
+	if (written < 0) {
+		mylog(LOG_ERROR, "bip_strcatf_fit: vsnprintf failed with: %s",
+		      strerror(errno));
+		return NULL;
+	}
+
+	if ((unsigned)written >= *remaining) {
+		mylog(LOG_DEBUGVERB,
+		      "bip_strcatf_fit,vsnprintf: no space left");
+		goto end;
+	}
+
+	res = memccpy(str, str2, '\0', *remaining);
+	if (!res) {
+		mylog(LOG_DEBUGTOOMUCH,
+		      "bip_strcatf_fit: memccpy() failed, "
+		      "remaining %lu",
+		      *remaining);
+		goto end;
+	}
+
+	if (res < str) {
+		mylog(LOG_DEBUG, "bip_strcatf_fit: memccpy res < str");
+		goto end;
+	}
+	res--;
+	(*remaining) -= (size_t)(res - str);
+
+end:
+	va_end(ap);
+	return res;
+}
+
+void bip_clock_gettime(clockid_t clockid, struct timespec *tp)
+{
+	int err = clock_gettime(clockid, tp);
+
+	if (err != 0)
+		fatal("clock_gettime: %s", strerror(errno));
+
+	if (tp->tv_sec < 0 || tp->tv_nsec < 0)
+		fatal("clock_gettime returned negative time");
+}
+
 /*
  * <nick> ::= <letter> { <letter> | <number> | <special> }
  * <special> ::= '-' | '[' | ']' | '\' | '`' | '^' | '{' | '}'
@@ -92,10 +199,10 @@ int is_valid_nick(char *str)
 		return 0;
 
 	tmp = str;
-	while (*tmp != '\0' && (isalnum(*tmp) || *tmp == '-' || *tmp == '[' ||
-			*tmp == ']' || *tmp == '\\' || *tmp == '`' ||
-			*tmp == '^' || *tmp == '{' || *tmp == '}' ||
-			*tmp == '|' || *tmp == '_' ))
+	while (*tmp != '\0'
+	       && (isalnum(*tmp) || *tmp == '-' || *tmp == '[' || *tmp == ']'
+		   || *tmp == '\\' || *tmp == '`' || *tmp == '^' || *tmp == '{'
+		   || *tmp == '}' || *tmp == '|' || *tmp == '_'))
 		tmp++;
 	return (*tmp == '\0');
 }
@@ -107,8 +214,8 @@ int is_valid_username(char *str)
 		return 0;
 
 	tmp = str;
-	while (*tmp != '\0' && *tmp != ' ' && *tmp != '\0' && *tmp != '\r' &&
-			*tmp != '\n')
+	while (*tmp != '\0' && *tmp != ' ' && *tmp != '\0' && *tmp != '\r'
+	       && *tmp != '\n')
 		tmp++;
 	return (*tmp == '\0');
 }
@@ -122,9 +229,7 @@ char *timestamp(void)
 	time(&tv);
 	tm = localtime(&tv);
 
-	snprintf(ts, 20, "%02d-%02d-%04d %02d:%02d:%02d", tm->tm_mday,
-			tm->tm_mon + 1, tm->tm_year + 1900, tm->tm_hour,
-			tm->tm_min, tm->tm_sec);
+	strftime(ts, (size_t)20, "%d-%m-%Y %H:%M:%S", tm);
 	return ts;
 }
 
@@ -137,9 +242,7 @@ char *hrtime(time_t s)
 		return "never";
 	tm = localtime(&s);
 
-	snprintf(ts, 20, "%02d-%02d-%04d %02d:%02d:%02d", tm->tm_mday,
-			tm->tm_mon + 1, tm->tm_year + 1900, tm->tm_hour,
-			tm->tm_min, tm->tm_sec);
+	strftime(ts, (size_t)20, "%d-%m-%Y %H:%M:%S", tm);
 	return ts;
 }
 
@@ -168,27 +271,27 @@ void _mylog(int level, char *fmt, va_list ap)
 		return;
 
 	switch (level) {
-		case LOG_FATAL:
-			prefix = "FATAL: ";
-			break;
-		case LOG_DEBUGVERB:
-			prefix = "DEBUGVERB: ";
-			break;
-		case LOG_DEBUG:
-			prefix = "DEBUG: ";
-			break;
-		case LOG_ERROR:
-			prefix = "ERROR: ";
-			break;
-		case LOG_WARN:
-			prefix = "WARNING: ";
-			break;
-		case LOG_INFO:
-			prefix = "";
-			break;
-		default:
-			prefix = "";
-			break;
+	case LOG_FATAL:
+		prefix = "FATAL: ";
+		break;
+	case LOG_DEBUGVERB:
+		prefix = "DEBUGVERB: ";
+		break;
+	case LOG_DEBUG:
+		prefix = "DEBUG: ";
+		break;
+	case LOG_ERROR:
+		prefix = "ERROR: ";
+		break;
+	case LOG_WARN:
+		prefix = "WARNING: ";
+		break;
+	case LOG_INFO:
+		prefix = "";
+		break;
+	default:
+		prefix = "";
+		break;
 	}
 
 	fprintf(conf_global_log_file, "%s %s", timestamp(), prefix);
@@ -210,11 +313,10 @@ void mylog(int level, char *fmt, ...)
 
 #ifdef HAVE_BACKTRACE
 #include <execinfo.h>
-
-void print_trace(void)
+void dump_trace(void)
 {
 	void *array[32];
-	size_t size;
+	int size;
 
 	size = backtrace(array, 32);
 	fflush(conf_global_log_file);
@@ -232,7 +334,7 @@ void fatal(char *fmt, ...)
 	va_end(ap);
 
 #ifdef HAVE_BACKTRACE
-	print_trace();
+	dump_trace();
 #endif
 
 	exit(200);
@@ -509,13 +611,13 @@ void hash_init(hash_t *h, int options)
 		switch (options) {
 		case HASH_NOCASE:
 			list_init(&h->lists[i],
-				(int (*)(const void*, const void*))
-					hash_item_nocase_cmp);
+				  (int (*)(const void *,
+					   const void *))hash_item_nocase_cmp);
 			break;
 		case HASH_DEFAULT:
 			list_init(&h->lists[i],
-				(int (*)(const void*,const void*))
-					hash_item_cmp);
+				  (int (*)(const void *,
+					   const void *))hash_item_cmp);
 			break;
 		default:
 			fatal("wrong hash option %d", options);
@@ -558,8 +660,9 @@ static unsigned char hash_func(const char *pkey)
 	char c;
 	unsigned long hash = 5381; /* 5381 & 0xff makes more sense */
 
+	// toupper should not return negative values (only char compatible int)
 	while ((c = *pkey++))
-		hash = ((hash << 5) + hash) ^ toupper(c);
+		hash = ((hash << 5) + hash) ^ (long unsigned)toupper(c);
 	return (unsigned char)hash;
 }
 
@@ -702,7 +805,7 @@ void hash_dump(hash_t *h)
 {
 	hash_iterator_t it;
 	assert(h);
-	for (hash_it_init(h, &it); hash_it_key(&it) ;hash_it_next(&it))
+	for (hash_it_init(h, &it); hash_it_key(&it); hash_it_next(&it))
 		printf("%s => %p\n", hash_it_key(&it), hash_it_item(&it));
 }
 
@@ -738,7 +841,7 @@ char *bip_strmaydup(char *s)
 void strucase(char *s)
 {
 	while (*s) {
-		*s = toupper(*s);
+		*s = (char)toupper(*s); // toupper, safe to cast to char
 		s++;
 	}
 }
@@ -768,15 +871,17 @@ void array_ensure(array_t *a, int index)
 
 	if (array_includes(a, index))
 		return;
-	a->elemv = bip_realloc(a->elemv, sizeof(void *) * (index + 1));
-	memset(a->elemv + a->elemc, 0, sizeof(void *) * (index + 1 - a->elemc));
+	a->elemv = bip_realloc(a->elemv, sizeof(void *) * (size_t)(index + 1));
+	// a->elemc should be lower than index + 1
+	memset(a->elemv + a->elemc, 0,
+	       sizeof(void *) * (size_t)(index + 1 - a->elemc));
 	a->elemc = index + 1;
 }
 
-void *array_drop(array_t *a, int index)
+const void *array_drop(array_t *a, int index)
 {
 	int i;
-	void *ret;
+	const void *ret;
 
 	assert(a && array_includes(a, index));
 
